import os

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class CiteULike(Dataset):

    __url__ = "http://konect.cc/files/download.tsv.citeulike-ut.tar.bz2"
    __corefile__ = os.path.join("citeulike-ut", "out.citeulike-ut")

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir)

    def transform(self):
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            sep=" ",
            header=None,
            index_col=False,
            names=["user_id", "tag_id", "positive", "timestamp"],
            usecols=[0, 1, 3],
            comment="%",
            converters={"timestamp": lambda x: int(float(x))},
        )
        df = df.rename(columns={"tag_id": "item_id"})
        return df



from datetime import datetime

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class TaFeng(Dataset):

    __url__ = "https://sites.google.com/site/dataminingcourse2009/spring2016/annoucement2016/assignment3/D11-02.ZIP"
    __corefile__ = ["D11", "D12", "D01", "D02"]

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir)

    def transform(self):
        dfs = []
        for cf in self.__corefile__:
            df = pd.read_csv(
                self.rootdir.joinpath(cf),
                sep=";",
                header=0,
                index_col=False,
                names=[
                    "timestamp",
                    "user_id",
                    "age",
                    "area",
                    "pcate",
                    "item_id",
                    "number",
                    "cost",
                    "price",
                ],
                usecols=[0, 1, 5],
                encoding="big5",
                converters={
                    "timestamp": lambda x: int(
                        datetime.strptime(x, "%Y-%m-%d %H:%M:%S").timestamp()
                    )
                },
            )
            dfs.append(df)
        return pd.concat(dfs, ignore_index=True)



import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url


class Amazon(Dataset):

    __corefile__ = {
        "Books": "ratings_Books.csv",
        "Electronics": "ratings_Electronics.csv",
        "Movies": "ratings_Movies_and_TV.csv",
        "CDs": "ratings_CDs_and_Vinyl.csv",
        "Clothing": "ratings_Clothing_Shoes_and_Jewelry.csv",
        "Home": "ratings_Home_and_Kitchen.csv",
        "Kindle": "ratings_Kindle_Store.csv",
        "Sports": "ratings_Sports_and_Outdoors.csv",
        "Phones": "ratings_Cell_Phones_and_Accessories.csv",
        "Health": "ratings_Health_and_Personal_Care.csv",
        "Toys": "ratings_Toys_and_Games.csv",
        "VideoGames": "ratings_Video_Games.csv",
        "Tools": "ratings_Tools_and_Home_Improvement.csv",
        "Beauty": "ratings_Beauty.csv",
        "Apps": "ratings_Apps_for_Android.csv",
        "Office": "ratings_Office_Products.csv",
        "Pet": "ratings_Pet_Supplies.csv",
        "Automotive": "ratings_Automotive.csv",
        "Grocery": "ratings_Grocery_and_Gourmet_Food.csv",
        "Patio": "ratings_Patio_Lawn_and_Garden.csv",
        "Baby": "ratings_Baby.csv",
        "Music": "ratings_Digital_Music.csv",
        "MusicalInstruments": "ratings_Musical_Instruments.csv",
        "InstantVideo": "ratings_Amazon_Instant_Video.csv",
    }

    url_prefix = "http://snap.stanford.edu/data/amazon/productGraph/categoryFiles/"

    def download(self, category):
        download_url(
            self.url_prefix + self.__corefile__[category],
            self.rootdir.joinpath(self.__corefile__[category]),
        )

    def transform(self, category, rating_threshold):
        """ Records with rating less than `rating_threshold` are dropped
        """
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__[category]),
            header=None,
            names=["user_id", "item_id", "rating", "timestamp"],
        )
        df = df[df["rating"] >= rating_threshold].drop("rating", axis=1)
        return df



import os

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class MovieLens20M(Dataset):

    __url__ = "http://files.grouplens.org/datasets/movielens/ml-20m.zip"
    __corefile__ = os.path.join("ml-20m", "ratings.csv")

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir)

    def transform(self, rating_threshold):
        """ Records with rating less than `rating_threshold` are dropped
        """
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            header=0,
            names=["user_id", "movie_id", "rating", "timestamp"],
        )
        df = df.rename(columns={"movie_id": "item_id"})
        df = df[df.rating >= rating_threshold].drop("rating", axis=1)
        return df



import json
import logging
import os
from pathlib import Path

from srdatasets.datasets import __datasets__, dataset_classes

logger = logging.getLogger(__name__)

__warehouse__ = Path(os.path.expanduser("~")).joinpath(".srdatasets")


def get_processed_datasets():
    P = [
        "dev/train.pkl",
        "dev/test.pkl",
        "dev/stats.json",
        "test/train.pkl",
        "test/test.pkl",
        "test/stats.json",
        "config.json",
    ]
    D = {}
    if __warehouse__.exists():
        for d in __warehouse__.iterdir():  # loop datasets
            if d.joinpath("processed").exists():
                configs = []
                for c in d.joinpath("processed").iterdir():  # loop configs
                    if all(c.joinpath(p).exists() for p in P):
                        configs.append(c.stem)
                if configs:
                    D[d.stem] = configs
    return D


def get_downloaded_datasets():
    """ Simple check based on the existences of corefiles
    """
    D = []
    for d in __datasets__:
        if "-" in d:
            corefile = dataset_classes[d.split("-")[0]].__corefile__[d.split("-")[1]]
        else:
            corefile = dataset_classes[d].__corefile__
        if isinstance(corefile, list):
            if all(__warehouse__.joinpath(d, "raw", cf).exists() for cf in corefile):
                D.append(d)
        else:
            if __warehouse__.joinpath(d, "raw", corefile).exists():
                D.append(d)
    return D


def read_json(path):
    content = {}
    if path.exists():
        with open(path, "r") as f:
            try:
                content = json.load(f)
            except:
                logger.exception("Read json file failed")
    return content


def get_datasetname(name):
    return {d.lower(): d for d in __datasets__}.get(name.lower(), name)



import copy
import logging
import math
import pickle
import random
from collections import Counter

import numpy as np

from srdatasets.datasets import __datasets__
from srdatasets.utils import (__warehouse__, get_datasetname,
                              get_processed_datasets)

logger = logging.getLogger(__name__)


class DataLoader:
    def __init__(
        self,
        dataset_name: str,
        config_id: str,
        batch_size: int = 1,
        train: bool = True,
        development: bool = False,
        negatives_per_target: int = 0,
        include_timestamp: bool = False,
        drop_last: bool = False,
    ):
        """Loader of sequential recommendation datasets

        Args:
            dataset_name (str): dataset name.
            config_id (str): dataset config id
            batch_size (int): batch_size
            train (bool, optional): load training data
            development (bool, optional): use the dataset for hyperparameter tuning
            negatives_per_target (int, optional): number of negative samples per target
            include_timestamp (bool, optional): add timestamps to batch data
            drop_last (bool, optional): drop last incomplete batch

        Note: training data is shuffled automatically.
        """
        dataset_name = get_datasetname(dataset_name)

        if dataset_name not in __datasets__:
            raise ValueError(
                "Unrecognized dataset, currently supported datasets: {}".format(
                    ", ".join(__datasets__)
                )
            )

        _processed_datasets = get_processed_datasets()
        if dataset_name not in _processed_datasets:
            raise ValueError(
                "{} has not been processed, currently processed datasets: {}".format(
                    dataset_name,
                    ", ".join(_processed_datasets) if _processed_datasets else "none",
                )
            )

        if config_id not in _processed_datasets[dataset_name]:
            raise ValueError(
                "Unrecognized config id, existing config ids: {}".format(
                    ", ".join(_processed_datasets[dataset_name])
                )
            )

        if negatives_per_target < 0:
            negatives_per_target = 0
            logger.warning(
                "Number of negative samples per target should >= 0, reset to 0"
            )

        if not train and negatives_per_target > 0:
            logger.warning(
                "Negative samples are used for training, set negatives_per_target has no effect when testing"
            )

        dataset_path = __warehouse__.joinpath(
            dataset_name,
            "processed",
            config_id,
            "dev" if development else "test",
            "train.pkl" if train else "test.pkl",
        )
        with open(dataset_path, "rb") as f:
            self.dataset = pickle.load(f)  # list

        if train:
            counter = Counter()
            for data in self.dataset:
                if len(data) > 5:
                    counter.update(data[1] + data[2] + data[3])
                else:
                    counter.update(data[1] + data[2])
            self.item_counts = np.array(
                [counter[i] for i in range(max(counter.keys()) + 1)]
            )

        if batch_size <= 0:
            raise ValueError("batch_size should >= 1")
        if batch_size > len(self.dataset):
            raise ValueError("batch_size exceeds the dataset size")

        self.batch_size = batch_size
        self.train = train
        self.include_timestamp = include_timestamp
        self.negatives_per_target = negatives_per_target
        self.drop_last = drop_last
        self._batch_idx = 0

    def __iter__(self):
        return self

    def __len__(self):
        """Number of batches
        """
        if self.drop_last:
            return math.floor(len(self.dataset) / self.batch_size)
        else:
            return math.ceil(len(self.dataset) / self.batch_size)

    def sample_negatives(self, batch_items_list):
        negatives = []
        for b in np.concatenate(batch_items_list, 1):
            item_counts = copy.deepcopy(self.item_counts)
            item_counts[b] = 0
            item_counts[0] = 0
            probs = item_counts / item_counts.sum()
            _negatives = np.random.choice(
                len(item_counts),
                size=self.negatives_per_target * batch_items_list[-1].shape[1],
                replace=False,
                p=probs,
            )
            _negatives = _negatives.reshape((-1, self.negatives_per_target))
            negatives.append(_negatives)
        return np.stack(negatives)

    def __next__(self):
        """
        Returns:
            user_ids: (batch_size,)
            input sequences: (batch_size, input_len)
            target sequences: (batch_size, target_len)
        """
        if self._batch_idx == len(self):
            self._batch_idx = 0
            raise StopIteration
        else:
            if self._batch_idx == 0 and self.train:
                random.shuffle(self.dataset)
            batch = self.dataset[
                self._batch_idx
                * self.batch_size : (self._batch_idx + 1)
                * self.batch_size
            ]
            self._batch_idx += 1
            batch_data = [np.array(b) for b in zip(*batch)]
            # Diff task
            target_idx = 3 if len(batch_data) > 5 else 2
            if not self.include_timestamp:
                batch_data = batch_data[: target_idx + 1]
            # Sampling negatives
            if self.train and self.negatives_per_target > 0:
                negatives = self.sample_negatives(batch_data[1 : target_idx + 1])
                batch_data.append(negatives)
            return batch_data



from srdatasets.datasets.amazon import Amazon
from srdatasets.datasets.citeulike import CiteULike
from srdatasets.datasets.foursquare import FourSquare
from srdatasets.datasets.gowalla import Gowalla
from srdatasets.datasets.lastfm1k import Lastfm1K
from srdatasets.datasets.movielens20m import MovieLens20M
from srdatasets.datasets.retailrocket import Retailrocket
from srdatasets.datasets.tafeng import TaFeng
from srdatasets.datasets.taobao import Taobao
from srdatasets.datasets.tmall import Tmall
from srdatasets.datasets.yelp import Yelp

dataset_classes = {
    "Amazon": Amazon,
    "CiteULike": CiteULike,
    "FourSquare": FourSquare,
    "Gowalla": Gowalla,
    "Lastfm1K": Lastfm1K,
    "MovieLens20M": MovieLens20M,
    "Retailrocket": Retailrocket,
    "TaFeng": TaFeng,
    "Taobao": Taobao,
    "Tmall": Tmall,
    "Yelp": Yelp,
}

amazon_datasets = ["Amazon-" + c for c in Amazon.__corefile__.keys()]
foursquare_datasets = ["FourSquare-" + c for c in FourSquare.__corefile__.keys()]

__datasets__ = (
    amazon_datasets
    + ["CiteULike"]
    + foursquare_datasets
    + [
        "Gowalla",
        "Lastfm1K",
        "MovieLens20M",
        "TaFeng",
        "Retailrocket",
        "Taobao",
        "Tmall",
        "Yelp",
    ]
)



import logging
import pickle
from collections import Counter

import torch
import torch.utils.data

from srdatasets.datasets import __datasets__
from srdatasets.utils import (__warehouse__, get_datasetname,
                              get_processed_datasets)

logger = logging.getLogger(__name__)


class Dataset(torch.utils.data.Dataset):
    def __init__(self, name: str, config_id: str, train: bool, development: bool):
        super(Dataset, self).__init__()
        datapath = __warehouse__.joinpath(
            name,
            "processed",
            config_id,
            "dev" if development else "test",
            "train.pkl" if train else "test.pkl",
        )
        if datapath.exists():
            with open(datapath, "rb") as f:
                self.dataset = pickle.load(f)
        else:
            raise ValueError("{} does not exist!".format(datapath))
        if train:
            self.item_counts = Counter()
            for data in self.dataset:
                if len(data) > 5:
                    self.item_counts.update(data[1] + data[2] + data[3])
                else:
                    self.item_counts.update(data[1] + data[2])

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, idx):
        return tuple(map(torch.tensor, self.dataset[idx]))


class DataLoader(torch.utils.data.DataLoader):
    def collate_fn(self, batch):
        """ Negative sampling and Timestamps removal or adding
        """
        target_pos = 3 if len(batch[0]) > 5 else 2

        batch_data = list(zip(*batch))
        if self.include_timestamp:
            batch_data = list(map(torch.stack, batch_data))
        else:
            batch_data = list(map(torch.stack, batch_data[: target_pos + 1]))

        if self.train and self.negatives_per_target > 0:
            batch_item_counts = self.item_counts.repeat(len(batch), 1).scatter(
                1, torch.cat(batch_data[1 : target_pos + 1], 1), 0
            )
            # Prevent padding item 0 from being negative samples
            batch_item_counts[:, 0] = 0
            negatives = torch.multinomial(
                batch_item_counts,
                self.negatives_per_target * batch_data[target_pos].size(1),
            )
            negatives = negatives.view(len(batch), -1, self.negatives_per_target)
            batch_data.append(negatives)
        return batch_data

    def __init__(
        self,
        dataset_name: str,
        config_id: str,
        batch_size: int = 1,
        train: bool = True,
        development: bool = False,
        negatives_per_target: int = 0,
        include_timestamp: bool = False,
        **kwargs,
    ):
        """Loader of sequential recommendation datasets

        Args:
            dataset_name (str): dataset name
            config_id (str): dataset config id
            batch_size (int): batch_size
            train (bool, optional): load training dataset
            development (bool, optional): use the dataset for hyperparameter tuning
            negatives_per_target (int, optional): number of negative samples per target
            include_timestamp (bool, optional): add timestamps to batch data
        
        Note: training data is shuffled automatically.
        """
        dataset_name = get_datasetname(dataset_name)

        if dataset_name not in __datasets__:
            raise ValueError(
                "Unrecognized dataset, currently supported datasets: {}".format(
                    ", ".join(__datasets__)
                )
            )

        _processed_datasets = get_processed_datasets()
        if dataset_name not in _processed_datasets:
            raise ValueError(
                "{} has not been processed, currently processed datasets: {}".format(
                    dataset_name,
                    ", ".join(_processed_datasets) if _processed_datasets else "none",
                )
            )

        if config_id not in _processed_datasets[dataset_name]:
            raise ValueError(
                "Unrecognized config id, existing config ids: {}".format(
                    ", ".join(_processed_datasets[dataset_name])
                )
            )

        if negatives_per_target < 0:
            negatives_per_target = 0
            logger.warning(
                "Number of negative samples per target should >= 0, reset to 0"
            )

        if not train and negatives_per_target > 0:
            logger.warning(
                "Negative samples are used for training, set negatives_per_target has no effect when testing"
            )

        self.train = train
        self.include_timestamp = include_timestamp
        self.negatives_per_target = negatives_per_target

        _dataset = Dataset(dataset_name, config_id, train, development)
        if train:
            self.item_counts = torch.tensor(
                [
                    _dataset.item_counts[i]
                    for i in range(max(_dataset.item_counts.keys()) + 1)
                ],
                dtype=torch.float,
            )

        super(DataLoader, self).__init__(
            _dataset,
            batch_size=batch_size,
            shuffle=train,
            collate_fn=self.collate_fn,
            **kwargs,
        )



import os

from srdatasets.utils import __warehouse__
from srdatasets.datasets import dataset_classes


def _download(dataset_name):
    _rawdir = __warehouse__.joinpath(dataset_name, "raw")
    os.makedirs(_rawdir, exist_ok=True)

    if dataset_name.startswith("Amazon"):
        dataset_classes["Amazon"](_rawdir).download(dataset_name.split("-")[1])
    elif dataset_name.startswith("FourSquare"):
        dataset_classes["FourSquare"](_rawdir).download()
    else:
        dataset_classes[dataset_name](_rawdir).download()



import argparse
import logging
import sys

from pandas.io.json import json_normalize
from tabulate import tabulate

from srdatasets.datasets import __datasets__
from srdatasets.download import _download
from srdatasets.process import _process
from srdatasets.utils import (__warehouse__, get_datasetname,
                              get_downloaded_datasets, get_processed_datasets,
                              read_json)

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s %(levelname)s %(filename)s:%(lineno)d] %(message)s",
    datefmt="%m/%d/%Y %H:%M:%S",
)


def main():
    parser = argparse.ArgumentParser("srdatasets | python -m srdatasets")
    subparsers = parser.add_subparsers(help="commands", dest="command")
    # info
    parser_i = subparsers.add_parser("info", help="print local datasets info")
    parser_i.add_argument("--dataset", type=str, default=None, help="dataset name")

    # download
    parser_d = subparsers.add_parser("download", help="download datasets")
    parser_d.add_argument("--dataset", type=str, required=True, help="dataset name")

    # process
    parser_g = subparsers.add_parser(
        "process",
        help="process datasets",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser_g.add_argument("--dataset", type=str, required=True, help="dataset name")
    parser_g.add_argument(
        "--min-freq-item", type=int, default=5, help="minimum occurrence times of item"
    )
    parser_g.add_argument(
        "--min-freq-user", type=int, default=5, help="minimum occurrence times of user"
    )
    parser_g.add_argument(
        "--task",
        type=str,
        choices=["short", "long-short"],
        default="short",
        help="short-term task or long-short-term task",
    )
    parser_g.add_argument(
        "--split-by",
        type=str,
        choices=["user", "time"],
        default="user",
        help="user-based or time-based dataset splitting",
    )
    parser_g.add_argument(
        "--dev-split",
        type=float,
        default=0.1,
        help="[user-split] the fraction of developemnt dataset",
    )
    parser_g.add_argument(
        "--test-split",
        type=float,
        default=0.2,
        help="[user-split] the fraction of test dataset",
    )
    parser_g.add_argument(
        "--input-len", type=int, default=5, help="[short] input sequence length"
    )
    parser_g.add_argument(
        "--target-len", type=int, default=1, help="target sequence length"
    )
    parser_g.add_argument(
        "--no-augment", action="store_true", help="do not use data augmentation"
    )
    parser_g.add_argument(
        "--remove-duplicates",
        action="store_true",
        help="remove duplicate items in user sequence",
    )
    parser_g.add_argument(
        "--session-interval",
        type=int,
        default=0,
        help="[short-optional, long-short-required] split user sequences into sessions (minutes)",
    )
    parser_g.add_argument(
        "--max-session-len", type=int, default=20, help="max session length"
    )
    parser_g.add_argument(
        "--min-session-len", type=int, default=2, help="min session length"
    )
    parser_g.add_argument(
        "--pre-sessions",
        type=int,
        default=10,
        help="[long-short] number of previous sessions",
    )
    parser_g.add_argument(
        "--pick-targets",
        type=str,
        choices=["last", "random"],
        default="random",
        help="[long-short] pick T random or last items from current session as targets",
    )
    parser_g.add_argument(
        "--rating-threshold",
        type=int,
        default=4,
        help="[Amazon-X, Movielens20M, Yelp] ratings great or equal than this are treated as valid",
    )
    parser_g.add_argument(
        "--item-type",
        type=str,
        choices=["song", "artist"],
        default="song",
        help="[Lastfm1K] set item to song or artist",
    )
    args = parser.parse_args()

    if "dataset" in args and args.dataset is not None:
        args.dataset = get_datasetname(args.dataset)

    if args.command is None:
        parser.print_help()
    else:
        downloaded_datasets = get_downloaded_datasets()
        processed_datasets = get_processed_datasets()

        if args.command == "download":
            if args.dataset not in __datasets__:
                raise ValueError("Supported datasets: {}".format(", ".join(__datasets__)))
            if args.dataset in downloaded_datasets:
                raise ValueError("{} has been downloaded".format(args.dataset))
            _download(args.dataset)
        elif args.command == "process":
            if args.dataset not in __datasets__:
                raise ValueError("Supported datasets: {}".format(", ".join(__datasets__)))
            if args.dataset not in downloaded_datasets:
                raise ValueError("{} has not been downloaded".format(args.dataset))

            if args.split_by == "user":
                if args.dev_split <= 0 or args.dev_split >= 1:
                    raise ValueError("dev split ratio should be in (0, 1)")
                if args.test_split <= 0 or args.test_split >= 1:
                    raise ValueError("test split ratio should be in (0, 1)")

            if args.task == "short":
                if args.input_len <= 0:
                    raise ValueError("input length must > 0")
                if args.session_interval < 0:
                    raise ValueError("session interval must >= 0 minutes")
            else:
                if args.session_interval <= 0:
                    raise ValueError("session interval must > 0 minutes")
                if args.pre_sessions < 1:
                    raise ValueError("number of previous sessions must > 0")

            if args.target_len <= 0:
                raise ValueError("target length must > 0")

            if args.session_interval > 0:
                if args.min_session_len <= args.target_len:
                    raise ValueError("min session length must > target length")
                if args.max_session_len < args.min_session_len:
                    raise ValueError("max session length must >= min session length")

            if args.dataset in processed_datasets:
                # TODO Improve processed check when some arguments are not used
                time_splits = {}
                for c in processed_datasets[args.dataset]:
                    config = read_json(
                        __warehouse__.joinpath(args.dataset, "processed", c, "config.json")
                    )
                    if args.split_by == "user" and all(
                        [args.__dict__[k] == v for k, v in config.items()]
                    ):
                        print("You have run this config, the config id is: {}".format(c))
                        sys.exit(1)
                    if args.split_by == "time" and all(
                        [
                            args.__dict__[k] == v
                            for k, v in config.items()
                            if k not in ["dev_split", "test_split"]
                        ]
                    ):
                        time_splits[(config["dev_split"], config["test_split"])] = c
                args.time_splits = time_splits
            _process(args)
        else:
            if args.dataset is None:
                table = [
                    [
                        d,
                        "Y" if d in downloaded_datasets else "",
                        len(processed_datasets[d]) if d in processed_datasets else "",
                    ]
                    for d in __datasets__
                ]
                print(
                    tabulate(
                        table,
                        headers=["name", "downloaded", "processed configs"],
                        tablefmt="psql",
                    )
                )
            else:
                if args.dataset not in __datasets__:
                    raise ValueError(
                        "Supported datasets: {}".format(", ".join(__datasets__))
                    )
                if args.dataset not in downloaded_datasets:
                    print("{} has not been downloaded".format(args.dataset))
                else:
                    if args.dataset not in processed_datasets:
                        print("{} has not been processed".format(args.dataset))
                    else:
                        configs = json_normalize(
                            [
                                read_json(
                                    __warehouse__.joinpath(
                                        args.dataset, "processed", c, "config.json"
                                    )
                                )
                                for c in processed_datasets[args.dataset]
                            ]
                        )
                        print("Configs")
                        configs_part1 = configs.iloc[:, :8]
                        configs_part1.insert(
                            0, "config id", processed_datasets[args.dataset]
                        )
                        print(
                            tabulate(
                                configs_part1,
                                headers="keys",
                                showindex=False,
                                tablefmt="psql",
                            )
                        )
                        print()
                        configs_part2 = configs.iloc[:, 8:]
                        configs_part2.insert(
                            0, "config id", processed_datasets[args.dataset]
                        )
                        print(
                            tabulate(
                                configs_part2,
                                headers="keys",
                                showindex=False,
                                tablefmt="psql",
                            )
                        )
                        print("\nStats")
                        stats = json_normalize(
                            [
                                read_json(
                                    __warehouse__.joinpath(
                                        args.dataset, "processed", c, m, "stats.json"
                                    )
                                )
                                for c in processed_datasets[args.dataset]
                                for m in ["dev", "test"]
                            ]
                        )
                        modes = ["development", "test"] * len(
                            processed_datasets[args.dataset]
                        )
                        stats.insert(0, "mode", modes)
                        ids = []
                        for c in processed_datasets[args.dataset]:
                            ids.extend([c, ""])
                        stats.insert(0, "config id", ids)
                        print(
                            tabulate(
                                stats, headers="keys", showindex=False, tablefmt="psql"
                            )
                        )


if __name__ == "__main__":
    main()



from datetime import datetime

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class Gowalla(Dataset):

    __url__ = "https://snap.stanford.edu/data/loc-gowalla_totalCheckins.txt.gz"
    __corefile__ = "loc-gowalla_totalCheckins.txt"

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir.joinpath("loc-gowalla_totalCheckins.txt"))

    def transform(self):
        """ Time: yyyy-mm-ddThh:mm:ssZ -> timestamp """
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            sep="\t",
            names=["user_id", "check_in_time", "latitude", "longtitude", "location_id"],
            usecols=[0, 1, 4],
            converters={
                "check_in_time": lambda x: int(
                    datetime.strptime(x, "%Y-%m-%dT%H:%M:%SZ").timestamp()
                )
            },
        )
        df = df.rename(columns={"location_id": "item_id", "check_in_time": "timestamp"})
        return df



import os
from abc import ABC, abstractmethod
from urllib.parse import urlparse


class Dataset(ABC):
    """ Base dataset of SR datasets
    """

    def __init__(self, rootdir):
        """ `rootdir` is the directory of the raw dataset """
        self.rootdir = rootdir

    @property
    def rawpath(self):
        if hasattr(self, "__url__"):
            return self.rootdir.joinpath(os.path.basename(urlparse(self.__url__).path))
        else:
            return ""

    @abstractmethod
    def download(self):
        """ Download and extract the raw dataset """
        pass

    @abstractmethod
    def transform(self):
        """ Transform to the general data format, which is
        a pd.DataFrame instance that contains three columns: [user_id, item_id, timestamp]
        """
        pass



import logging
import os
from datetime import datetime

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import extract

logger = logging.getLogger(__name__)


class Tmall(Dataset):

    __corefile__ = os.path.join("data_format1", "user_log_format1.csv")

    def download(self):
        if not self.rootdir.joinpath("data_format1.zip").exists():
            logger.warning(
                "Since Tmall dataset is not directly accessible, please visit https://tianchi.aliyun.com/dataset/dataDetail?dataId=47 and download it manually, after downloaded, place file 'data_format1.zip' under %s and run this command again",
                self.rootdir,
            )
        else:
            extract(self.rootdir.joinpath("data_format1.zip"), self.rootdir)

    def transform(self):
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            header=0,
            index_col=False,
            usecols=[0, 1, 5],
            converters={
                "time_stamp": lambda x: int(
                    datetime.strptime("2015" + x, "%Y%m%d").timestamp()
                )
            },
        )
        df = df.rename(columns={"time_stamp": "timestamp"})
        return df



import json
import logging
import math
import os
import pickle
import random
import sys
import time
from collections import defaultdict
from datetime import datetime

from tqdm import tqdm

from srdatasets.datasets import dataset_classes
from srdatasets.utils import __warehouse__


logger = logging.getLogger(__name__)


def _process(args):
    if "-" in args.dataset:
        classname, sub = args.dataset.split("-")
    else:
        classname = args.dataset
    d = dataset_classes[classname](__warehouse__.joinpath(args.dataset, "raw"))

    config = {
        "min_freq_user": args.min_freq_user,
        "min_freq_item": args.min_freq_item,
        "input_len": args.input_len,
        "target_len": args.target_len,
        "no_augment": args.no_augment,
        "remove_duplicates": args.remove_duplicates,
        "session_interval": args.session_interval,
        "min_session_len": args.min_session_len,
        "max_session_len": args.max_session_len,
        "split_by": args.split_by,
        "dev_split": args.dev_split,
        "test_split": args.test_split,
        "task": args.task,
        "pre_sessions": args.pre_sessions,
        "pick_targets": args.pick_targets,
    }
    if classname in ["Amazon", "MovieLens20M", "Yelp"]:
        config["rating_threshold"] = args.rating_threshold
    elif classname == "Lastfm1K":
        config["item_type"] = args.item_type

    logger.info("Transforming...")
    if classname == "Amazon":
        df = d.transform(sub, args.rating_threshold)
    elif classname in ["MovieLens20M", "Yelp"]:
        df = d.transform(args.rating_threshold)
    elif classname == "FourSquare":
        df = d.transform(sub)
    elif classname == "Lastfm1K":
        df = d.transform(args.item_type)
    else:
        df = d.transform()

    if args.split_by == "time":
        config["dev_split"], config["test_split"] = access_split_days(df)
        # Processed check
        if (
            "time_splits" in args
            and (config["dev_split"], config["test_split"]) in args.time_splits
        ):
            logger.warning(
                "You have run this config, the config id is {}".format(
                    args.time_splits[(config["dev_split"], config["test_split"])]
                )
            )
            sys.exit(1)
        config["max_timestamp"] = df["timestamp"].max()

    preprocess_and_save(df, args.dataset, config)


def access_split_days(df):
    min_timestamp = df["timestamp"].min()
    max_timestamp = df["timestamp"].max()
    first_day = datetime.fromtimestamp(min_timestamp).strftime("%Y-%m-%d")
    last_day = datetime.fromtimestamp(max_timestamp).strftime("%Y-%m-%d")
    total_days = math.ceil((max_timestamp - min_timestamp) / 86400)
    print("Date range: {} ~ {}, total days: {}".format(first_day, last_day, total_days))
    while True:
        try:
            test_last_days = int(input("Last N days for test: "))
            dev_last_days = int(input("Last N days for dev: "))
            if test_last_days <= 0 or dev_last_days <= 0:
                raise ValueError
            elif test_last_days + dev_last_days >= total_days:
                raise AssertionError
            else:
                break
        except ValueError:
            print("Please input a positive integer!")
        except AssertionError:
            print("test_last_days + dev_last_days < total_days")
    return dev_last_days, test_last_days


def preprocess_and_save(df, dname, config):
    """General preprocessing method

    Args:
        df (DataFrame): columns: `user_id`, `item_id`, `timestamp`.
        args (Namespace): arguments.
    """
    # Generate sequences
    logger.info("Generating user sequences...")
    seqs = generate_sequences(df, config)

    # Split sequences in different ways
    if config["session_interval"] > 0:
        split = split_sequences_session
    else:
        split = split_sequences

    logger.info("Splitting user sequences into train/test...")
    train_seqs, test_seqs = split(seqs, config, 0)

    logger.info("Splitting train into dev-train/dev-test...")
    dev_train_seqs, dev_test_seqs = split(train_seqs, config, 1)

    # Remove duplicates (optional)
    if config["remove_duplicates"]:
        logger.info("Removing duplicates...")
        train_seqs, test_seqs, dev_train_seqs, dev_test_seqs = [
            remove_duplicates(seqs, config)
            for seqs in [train_seqs, test_seqs, dev_train_seqs, dev_test_seqs]
        ]

    # Do not use data augmentation (optional)
    if config["no_augment"]:
        logger.info("Enabling no data augmentation...")
        train_seqs, test_seqs, dev_train_seqs, dev_test_seqs = [
            enable_no_augment(seqs, config)
            for seqs in [train_seqs, test_seqs, dev_train_seqs, dev_test_seqs]
        ]

    # Remove unknowns
    logger.info("Removing unknowns in test...")
    test_seqs = remove_unknowns(train_seqs, test_seqs, config)

    logger.info("Removing unknowns in dev-test...")
    dev_test_seqs = remove_unknowns(dev_train_seqs, dev_test_seqs, config)

    # Reassign user and item ids
    logger.info("Reassigning ids (train/test)...")
    train_seqs, test_seqs = reassign_ids(train_seqs, test_seqs)

    logger.info("Reassigning ids (dev-train/dev-test)...")
    dev_train_seqs, dev_test_seqs = reassign_ids(dev_train_seqs, dev_test_seqs)

    # Make datasets based on task
    if config["task"] == "short":
        make_dataset = make_dataset_short
    else:
        make_dataset = make_dataset_long_short

    logger.info("Making datasets...")
    train_data, test_data, dev_train_data, dev_test_data = [
        make_dataset(seqs, config)
        for seqs in [train_seqs, test_seqs, dev_train_seqs, dev_test_seqs]
    ]

    # Dump to disk
    logger.info("Dumping...")
    processed_path = __warehouse__.joinpath(
        dname, "processed", "c" + str(int(time.time() * 1000))
    )
    dump(processed_path, train_data, test_data, 0)
    dump(processed_path, dev_train_data, dev_test_data, 1)

    # Save config
    save_config(processed_path, config)
    logger.info("OK")


def enable_no_augment(seqs, config):
    """
    For short-term task: keep most recent (input_len + target_len) items,
    For long-short-term task: keep most recent (pre_sessions + 1) sessions
    """
    seqs_ = []
    if config["task"] == "short":
        for user_id, seq in tqdm(seqs):
            seqs_.append((user_id, seq[-config["input_len"] - config["target_len"] :]))
    else:
        user_sessions = defaultdict(list)
        for user_id, seq in seqs:
            user_sessions[user_id].append(seq)
        for user_id, sessions in tqdm(user_sessions.items()):
            seqs_.extend((user_id, s) for s in sessions[-config["pre_sessions"] - 1])
    return seqs_


def reassign_ids(train_seqs, test_seqs):
    user_to_idx = {}
    item_to_idx = {}  # starts from 1, 0 for padding
    train_seqs_ = []
    test_seqs_ = []
    for user_id, seq in tqdm(train_seqs):
        # Build dicts
        if user_id not in user_to_idx:
            user_to_idx[user_id] = len(user_to_idx)
        for item, timestamp in seq:
            if item not in item_to_idx:
                item_to_idx[item] = len(item_to_idx) + 1
        # Reassign
        train_seqs_.append(
            (user_to_idx[user_id], [(item_to_idx[i], t) for i, t in seq])
        )
    for user_id, seq in tqdm(test_seqs):
        test_seqs_.append((user_to_idx[user_id], [(item_to_idx[i], t) for i, t in seq]))
    return train_seqs_, test_seqs_


def generate_sequences(df, config):
    logger.warning("Dropping items (freq < {})...".format(config["min_freq_item"]))
    df = drop_items(df, config["min_freq_item"])

    logger.warning("Dropping users (freq < {})...".format(config["min_freq_user"]))
    df = drop_users(df, config["min_freq_user"])

    logger.info("Grouping items by user...")
    df = df.sort_values("timestamp", ascending=True)
    df["item_and_time"] = list(zip(df["item_id"], df["timestamp"]))
    seqs = df.groupby("user_id")["item_and_time"].apply(list)
    seqs = list(zip(seqs.index, seqs))

    logger.warning("Dropping too short user sequences...")
    seqs = [s for s in tqdm(seqs) if len(s[1]) > config["target_len"]]

    if config["session_interval"] > 0:
        logger.info("Splitting sessions...")
        _seqs = []
        for user_id, seq in tqdm(seqs):
            seq_buffer = []
            for i, (item_id, timestamp) in enumerate(seq):
                if i == 0:
                    seq_buffer.append((item_id, timestamp))
                else:
                    if timestamp - seq[i - 1][1] > config["session_interval"] * 60:
                        if len(seq_buffer) >= config["min_session_len"]:
                            _seqs.append(
                                (user_id, seq_buffer[-config["max_session_len"] :])
                            )
                        seq_buffer = [(item_id, timestamp)]
                    else:
                        seq_buffer.append((item_id, timestamp))
            if len(seq_buffer) >= config["min_session_len"]:
                _seqs.append((user_id, seq_buffer[-config["max_session_len"] :]))
        seqs = _seqs
    return seqs


def split_sequences(user_seq, config, mode):
    """ Without sessions
    """
    if config["split_by"] == "user":
        test_ratio = config["dev_split"] if mode else config["test_split"]
    else:
        last_days = (
            config["dev_split"] + config["test_split"] if mode else config["test_split"]
        )
        split_timestamp = config["max_timestamp"] - last_days * 86400
    train_seqs = []
    test_seqs = []
    for user_id, seq in tqdm(user_seq):
        train_num = 0
        if config["split_by"] == "user":
            train_num = math.ceil(len(seq) * (1 - test_ratio))
        else:
            for item, timestamp in seq:
                if timestamp < split_timestamp:
                    train_num += 1
        if train_num > config["target_len"]:
            train_seqs.append((user_id, seq[:train_num]))
            if len(seq) - train_num > config["target_len"]:
                test_seqs.append((user_id, seq[train_num:]))
    return train_seqs, test_seqs


def split_sequences_session(user_seq, config, mode):
    """ With sessions, when number of sessions is small, len of test_seqs can be 0
    """
    if config["split_by"] == "user":
        test_ratio = config["dev_split"] if mode else config["test_split"]
    else:
        last_days = (
            config["dev_split"] + config["test_split"] if mode else config["test_split"]
        )
        split_timestamp = config["max_timestamp"] - last_days * 86400
    user_sessions = defaultdict(list)
    for user_id, seq in user_seq:
        user_sessions[user_id].append(seq)
    train_seqs = []
    test_seqs = []
    for user_id, sessions in tqdm(user_sessions.items()):
        if config["split_by"] == "user":
            train_num = math.ceil((1 - test_ratio) * len(sessions))
        else:
            train_num = 0
            for s in sessions:
                if s[0][1] < split_timestamp:
                    train_num += 1
        if train_num > 0:
            train_seqs.extend((user_id, s) for s in sessions[:train_num])
            test_seqs.extend((user_id, s) for s in sessions[train_num:])
    return train_seqs, test_seqs


def remove_duplicates(user_seq, config):
    """ By default, we keep the first
    """
    user_seq_ = []
    for user_id, seq in tqdm(user_seq):
        seq_ = []
        shown_items = set()
        for item, timestamp in seq:
            if item not in shown_items:
                shown_items.add(item)
                seq_.append((item, timestamp))
        if config["session_interval"] > 0:
            if len(seq_) >= config["min_session_len"]:
                user_seq_.append((user_id, seq_))
        else:
            if len(seq_) > config["target_len"]:
                user_seq_.append((user_id, seq_))
    return user_seq_


def remove_unknowns(train_seqs, test_seqs, config):
    """ Remove users and items in test_seqs that are not shown in train_seqs
    """
    users = set()
    items = set()
    for user_id, seq in train_seqs:
        users.add(user_id)
        items.update([i for i, t in seq])
    test_seqs_ = []
    for user_id, seq in tqdm(test_seqs):
        if user_id in users:
            seq_ = [(i, t) for i, t in seq if i in items]
            if config["session_interval"] > 0:
                if len(seq_) >= config["min_session_len"]:
                    test_seqs_.append((user_id, seq_))
            else:
                if len(seq_) > config["target_len"]:
                    test_seqs_.append((user_id, seq_))
    return test_seqs_


def make_targets(seq, config):
    """ For long-short-term task
    """
    if config["pick_targets"] == "random":
        indices = list(range(len(seq)))
        random.shuffle(indices)
        cur_session_indices = sorted(indices[config["target_len"] :])
        target_indices = sorted(indices[: config["target_len"]])
        cur_session = [seq[i] for i in cur_session_indices]
        targets = [seq[i] for i in target_indices]
    else:
        cur_session = seq[: -config["target_len"]]
        targets = seq[-config["target_len"] :]
    # Padding
    cur_session = [(0, -1)] * (
        config["max_session_len"] - config["target_len"] - len(cur_session)
    ) + cur_session
    return cur_session, targets


def make_dataset_long_short(user_seq, config):
    """
        len of pre_sessions: max_session_len * pre_sessions
        len of cur_session: max_session_len - target_len
    """
    max_session_len = config["max_session_len"]
    n_pre_sessions = config["pre_sessions"]
    dataset = []
    user_sessions = defaultdict(list)
    for user_id, seq in user_seq:
        user_sessions[user_id].append(seq)
    for user_id, sessions in tqdm(user_sessions.items()):
        d = len(sessions) - 1 - n_pre_sessions
        if d <= 0:
            pre_sessions = [(0, -1)] * max_session_len * (-d)
            for s in sessions[:-1]:
                pre_sessions += [(0, -1)] * (max_session_len - len(s)) + s
            cur_session, targets = make_targets(sessions[-1], config)
            dataset.append((user_id, pre_sessions, cur_session, targets))
        else:
            for i in range(d):
                pre_sessions = []
                for s in sessions[i : i + n_pre_sessions]:
                    pre_sessions += [(0, -1)] * (max_session_len - len(s)) + s
                cur_session, targets = make_targets(
                    sessions[i + n_pre_sessions], config
                )
                dataset.append((user_id, pre_sessions, cur_session, targets))
    dataset_ = []
    for data in dataset:
        pre_items, pre_times = list(zip(*data[1]))
        cur_items, cur_times = list(zip(*data[2]))
        target_items, target_times = list(zip(*data[3]))
        dataset_.append(
            (
                data[0],
                pre_items,
                cur_items,
                target_items,
                pre_times,
                cur_times,
                target_times,
            )
        )
    return dataset_


def make_dataset_short(user_seq, config):
    """ Build dataset for short-term task
    """
    input_len = config["input_len"]
    target_len = config["target_len"]
    dataset = []
    for user_id, seq in tqdm(user_seq):
        if len(seq) <= input_len + target_len:
            padding_num = input_len + target_len - len(seq)
            dataset.append(
                (
                    user_id,
                    [(0, -1)] * padding_num + seq[:-target_len],
                    seq[-target_len:],
                )
            )
        else:
            augmented_seqs = [
                (
                    user_id,
                    seq[i : i + input_len],
                    seq[i + input_len : i + input_len + target_len],
                )
                for i in range(len(seq) - input_len - target_len + 1)
            ]
            dataset.extend(augmented_seqs)
    dataset_ = []
    for data in dataset:
        input_items, input_times = list(zip(*data[1]))
        target_items, target_times = list(zip(*data[2]))
        dataset_.append((data[0], input_items, target_items, input_times, target_times))
    return dataset_


def cal_stats(train_data, test_data):
    users = set()
    items = set()
    interactions = 0
    for data in train_data:
        users.add(data[0])
        if len(data) > 5:
            items_ = data[1] + data[2] + data[3]
        else:
            items_ = data[1] + data[2]
        for item in items_:
            if item > 0:  # reassigned
                items.add(item)
                interactions += 1
    stats = {
        "users": len(users),
        "items": len(items),
        "interactions": interactions,
        "density": interactions / len(users) / len(items),
        "train size": len(train_data),
        "test size": len(test_data),
    }
    return stats


def drop_users(df, min_freq):
    counts = df["user_id"].value_counts()
    df = df[df["user_id"].isin(counts[counts >= min_freq].index)]
    return df


def drop_items(df, min_freq):
    counts = df["item_id"].value_counts()
    df = df[df["item_id"].isin(counts[counts >= min_freq].index)]
    return df


def save_config(path, config):
    if "max_timestamp" in config:
        del config["max_timestamp"]
    with open(path.joinpath("config.json"), "w") as f:
        json.dump(config, f)


def dump(path, train_data, test_data, mode):
    """ Save preprocessed datasets """
    dirname = "dev" if mode else "test"
    os.makedirs(path.joinpath(dirname))
    with open(path.joinpath(dirname, "train.pkl"), "wb") as f:
        pickle.dump(train_data, f)
    with open(path.joinpath(dirname, "test.pkl"), "wb") as f:
        pickle.dump(test_data, f)
    stats = cal_stats(train_data, test_data)
    with open(path.joinpath(dirname, "stats.json"), "w") as f:
        json.dump(stats, f)


# ====== TODO API for custom dataset ====== #



import logging

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import extract

logger = logging.getLogger(__name__)


class Taobao(Dataset):

    __corefile__ = "UserBehavior.csv"

    def download(self):
        if not self.rootdir.joinpath("UserBehavior.csv.zip").exists():
            logger.warning(
                "Since Taobao dataset is not directly accessible, please visit https://tianchi.aliyun.com/dataset/dataDetail?dataId=649 and download it manually, after downloaded, place file 'UserBehavior.csv.zip' under %s and run this command again",
                self.rootdir,
            )
        else:
            extract(self.rootdir.joinpath("UserBehavior.csv.zip"), self.rootdir)

    def transform(self):
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            header=None,
            index_col=False,
            names=["user_id", "item_id", "category_id", "behavior_type", "timestamp"],
            usecols=[0, 1, 4],
        )
        return df



import os
from datetime import datetime

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class FourSquare(Dataset):

    __url__ = "http://www-public.imtbs-tsp.eu/~zhang_da/pub/dataset_tsmc2014.zip"

    __corefile__ = {
        "NYC": os.path.join("dataset_tsmc2014", "dataset_TSMC2014_NYC.txt"),
        "Tokyo": os.path.join("dataset_tsmc2014", "dataset_TSMC2014_TKY.txt"),
    }

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir)

    def transform(self, city):
        """ city: `NYC` or `Tokyo`
        """
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__[city]),
            sep="\t",
            header=None,
            names=[
                "user_id",
                "venue_id",
                "venue_category_id",
                "venue_category_name",
                "latitude",
                "longtitude",
                "timezone_offset",
                "utc_time",
            ],
            usecols=[0, 1, 7],
            converters={
                "utc_time": lambda x: int(
                    datetime.strptime(x, "%a %b %d %H:%M:%S %z %Y").timestamp()
                )
            },
        )
        df = df.rename(columns={"venue_id": "item_id", "utc_time": "timestamp"})
        return df



import os
from datetime import datetime

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import download_url, extract


class Lastfm1K(Dataset):

    __url__ = "http://mtg.upf.edu/static/datasets/last.fm/lastfm-dataset-1K.tar.gz"
    __corefile__ = os.path.join(
        "lastfm-dataset-1K", "userid-timestamp-artid-artname-traid-traname.tsv"
    )

    def download(self):
        download_url(self.__url__, self.rawpath)
        extract(self.rawpath, self.rootdir)

    def transform(self, item_type):
        """ item_type can be `artist` or `song`
        """
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            sep="\t",
            names=[
                "user_id",
                "timestamp",
                "artist_id",
                "artist_name",
                "song_id",
                "song_name",
            ],
            usecols=[0, 1, 2, 4],
            converters={
                "timestamp": lambda x: int(
                    datetime.strptime(x, "%Y-%m-%dT%H:%M:%SZ").timestamp()
                )
            },
        )
        if item_type == "song":
            df = df.drop("artist_id", axis=1).rename(columns={"song_id": "item_id"})
        else:
            df = df.drop("song_id", axis=1).rename(columns={"artist_id": "item_id"})
        return df



import logging

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import extract

logger = logging.getLogger(__name__)


class Retailrocket(Dataset):

    __corefile__ = "events.csv"

    def download(self):
        if not self.rootdir.joinpath("ecommerce-dataset.zip").exists():
            logger.warning(
                "Since RetailRocket dataset is not directly accessible, please visit https://www.kaggle.com/retailrocket/ecommerce-dataset and download it manually, after downloaded, place file 'ecommerce-dataset.zip' under %s and run this command again",
                self.rootdir,
            )
        else:
            extract(self.rootdir.joinpath("ecommerce-dataset.zip"), self.rootdir)

    def transform(self):
        df = pd.read_csv(
            self.rootdir.joinpath(self.__corefile__),
            header=0,
            index_col=False,
            usecols=[0, 1, 3],
            converters={"timestamp": lambda x: int(int(x) / 1000)},
        )
        df = df.rename(columns={"visitorid": "user_id", "itemid": "item_id"})
        return df



from setuptools import find_packages, setup

with open("README.md", "r") as f:
    long_description = f.read()

with open("requirements.txt", "r") as f:
    install_requires = f.read().splitlines()

setup(
    name="srdatasets",
    version="0.1.4",
    author="Cheng Guo",
    author_email="guocheng672@gmail.com",
    description="A collection of research ready datasets for sequential recommendation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/guocheng2018/sequential-recommendation-datasets",
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
    ],
    entry_points={"console_scripts": ["srdatasets=srdatasets.__main__:main"]},
)



import logging

import pandas as pd

from srdatasets.datasets.dataset import Dataset
from srdatasets.datasets.utils import extract

logger = logging.getLogger(__name__)


class Yelp(Dataset):

    __corefile__ = "review.json"

    def download(self):
        if not self.rootdir.joinpath("yelp_dataset.tar.gz").exists():
            logger.warning(
                "Since Yelp dataset is not directly accessible, please visit https://www.yelp.com/dataset/download and download it manually, after downloaded, place file 'yelp_dataset.tar.gz' under %s and run this command again",
                self.rootdir,
            )
        else:
            extract(self.rootdir.joinpath("yelp_dataset.tar.gz"), self.rootdir)

    def transform(self, stars_threshold):
        df = pd.read_json(
            self.rootdir.joinpath(self.__corefile__), orient="records", lines=True
        )
        df = df[["user_id", "business_id", "stars", "date"]]
        df["date"] = df["date"].apply(lambda x: int(x.timestamp()))
        df = df[df["stars"] >= stars_threshold].drop("stars", axis=1)
        df = df.rename(columns={"business_id": "item_id", "date": "timestamp"})
        return df



import bz2
import gzip
import logging
import os
import shutil
import tarfile
import urllib.request
from zipfile import ZipFile

from tqdm import tqdm

logger = logging.getLogger(__name__)


class DownloadProgressBar(tqdm):
    """ From https://stackoverflow.com/a/53877507/8810037
    """

    def update_to(self, b=1, bsize=1, tsize=None):
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)


def download_url(url, output_path):
    try:
        with DownloadProgressBar(
            unit="B", unit_scale=True, miniters=1, desc=url.split("/")[-1]
        ) as t:
            urllib.request.urlretrieve(
                url, filename=output_path, reporthook=t.update_to
            )
        logger.info("Download successful")
    except:
        logger.exception("Download failed, please try again")
        if output_path.exists():
            os.remove(output_path)


def extract(filepath, out):
    """ out: a file or a directory
    """
    logger.info("Unzipping...")
    filename = filepath.as_posix()

    if filename.endswith(".zip") or filename.endswith(".ZIP"):
        with ZipFile(filepath) as zipObj:
            zipObj.extractall(out)

    elif filename.endswith(".tar.gz"):
        with tarfile.open(filepath, "r:gz") as tar:
            tar.extractall(out)

    elif filename.endswith(".tar.bz2"):
        with tarfile.open(filepath, "r:bz2") as tar:
            tar.extractall(out)

    elif filename.endswith(".gz"):
        with gzip.open(filepath, "rb") as fin:
            with open(out, "wb") as fout:
                shutil.copyfileobj(fin, fout)

    elif filename.endswith(".bz2"):
        with bz2.open(filepath, "rb") as fin:
            with open(out, "wb") as fout:
                shutil.copyfileobj(fin, fout)
    else:
        logger.error("Unrecognized compressing format of %s", filepath)
        return

    logger.info("OK")
